@@ -1,5 +1,5 @@
 /* GAdmin-OpenVPN - An easy to use GTK+ frontend for the openvpn client.
- * Copyright (C) 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2008 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,11 +35,10 @@
 #include "populate_client_settings.h"
 #include "populate_conf_tab.h"
 
-
 extern gchar *selected_connection;
-
 extern int activated;
 
+int add_new_connection = 0;
 
 
 int has_value(gchar *input)
@@ -110,28 +109,86 @@ void add_firestarter_user_pre()
 }
 
 
+/* Does the connection name exist in the connection treeview */
+int connection_exists(struct w *widgets, gchar *name)
+{
+    GtkTreeSelection *selection;
+    gchar *temp_name = NULL;
+    GtkTreeIter iter;
+
+    if( widgets->connection_store )
+    {
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widgets->connection_treeview));
+	if( ! selection )
+	  return 0;
+
+        if( gtk_tree_model_get_iter_first(GTK_TREE_MODEL(widgets->connection_store), &iter) )
+        {
+            do {
+                    gtk_tree_model_get(GTK_TREE_MODEL(widgets->connection_store), &iter, 0, &temp_name, -1);
+
+		    if( name!=NULL && temp_name!=NULL )
+		    {
+			if( strcmp(name, temp_name) == 0 )
+			{
+			    if( temp_name!=NULL )
+			      g_free(temp_name);
+
+			    return 1;
+			}
+		    }
+		    if( temp_name!=NULL )
+		      g_free(temp_name);
+
+                }  while(gtk_tree_model_iter_next(GTK_TREE_MODEL(widgets->connection_store), &iter));
+         }
+    }
+
+    return 0;
+}
+
+
+/* Add a new connection */
+void add_connection(struct w *widgets)
+{
+    add_new_connection = 1;
+
+    apply_client_settings(widgets);
+
+    add_new_connection = 0;
+}
+
+
 void apply_client_settings(struct w *widgets)
 {
     /* Change the client configuration and sysinit script. */
     FILE *fp;
     int i = 0;
     gint active_index;
-    gchar *info, *cmd, *new_name;
+    gchar *info, *cmd;
     gchar *sysinit_script_path, *sysinit_start_script;
     gchar *conf_data=NULL, *openvpn_client_conf=NULL;
     gchar *pass_file, *pass_lines;
-    // Use later if the OpenVPN server begins using encrypted passwords: const char *encrypted_pass;
-
+    // Use later if the OpenVPN server begins using encrypted passwords:
+    // const char *encrypted_pass;
     gchar *firestarter_start, *firestarter_stop;
     gchar *non_firestarter_start, *non_firestarter_stop;
     gchar *share_tunnel_with_lan, *start_at_boot_label;
 
     /* Entries */
-    G_CONST_RETURN gchar *connection_name;
+    gchar *connection_name;
     G_CONST_RETURN gchar *server_address;
     G_CONST_RETURN gchar *user;
     G_CONST_RETURN gchar *pass;
+    G_CONST_RETURN gchar *passpath;
+    G_CONST_RETURN gchar *mountcmd;
     G_CONST_RETURN gchar *proxy_address;
+    /* Cert entries */
+    G_CONST_RETURN gchar *cacert;
+    G_CONST_RETURN gchar *cert;
+    G_CONST_RETURN gchar *key;
+    G_CONST_RETURN gchar *dhkey;
+    G_CONST_RETURN gchar *takey;
 
     /* Spin buttons */
     G_CONST_RETURN gchar *server_port;
@@ -143,85 +200,175 @@ void apply_client_settings(struct w *widgets)
     gchar *cipher=NULL, *compression=NULL, *proxy_line=NULL;
 
     /* Entries */
-    connection_name = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[0]));
+    connection_name = g_strdup_printf("%s", gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[0])));
     server_address  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[1]));
     user  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[2]));
     pass  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[3]));
-    proxy_address = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[4]));
+    passpath = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[4]));
+    mountcmd = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[5]));
+    proxy_address = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[6]));
+    /* Cert entries */
+    cacert = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[7]));
+    cert   = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[8]));
+    key    = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[9]));
+    dhkey  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[10]));
+    takey  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_entry[11]));
 
-    /* Strip newline chars at the end of the connection name */
-    if( selected_connection == NULL || strlen(selected_connection) < 3
-    ||  connection_name     == NULL || strlen(connection_name)     < 3 )
+    /* Spin buttons */
+    server_port = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_spinbutton[0]));
+    log_level   = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_spinbutton[1]));
+    proxy_port  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_spinbutton[2]));
+
+    /* Make sure we have some valid connection settings */
+    if( connection_name == NULL || strlen(connection_name) < 3 )
     {
-	printf("The connection name is too short.\n");
+	info = g_strdup_printf("%s", _("The connection name is too short.\n"));
+	show_info(info);
+	g_free(info);
+	return;
+    }
+    if( server_address == NULL || strlen(server_address) < 3 )
+    {
+	info = g_strdup_printf("%s", _("The server address is too short.\n"));
+	show_info(info);
+	g_free(info);
+	return;
+    }
+    if( server_port == NULL || strcmp(server_port, "0") == 0 )
+    {
+	info = g_strdup_printf("%s", _("The server port is not valid.\n"));
+	show_info(info);
+	g_free(info);
+	return;
+    }
+    if( strlen(cacert) < 3 || strlen(cert)  < 3 || strlen(key) < 3
+    ||  strlen(dhkey)  < 3 || strlen(takey) < 3 )
+    {
+	info = g_strdup_printf("%s", _("Missing certificate or key setting.\n"));
+	show_info(info);
+	g_free(info);
 	return;
     }
 
-    new_name = g_strdup_printf("%s", connection_name);
+    if( ! add_new_connection )
+    {
+	if( selected_connection == NULL || strlen(selected_connection) < 3 )
+	{
+	    /* No connection in the connections tab,
+	       select the one from the connection name entry */
+	    if( selected_connection!=NULL )
+              g_free(selected_connection);
+	    selected_connection = g_strdup_printf("%s", connection_name);
+	}
+
+    }
 
     /* Check that the specified connection name is valid. */
-    for(i=0; new_name[i]!='\0'; i++)
+    for(i=0; connection_name[i]!='\0'; i++)
     {
 	/* Change spaces to "-" chars */
-	if( new_name[i]==' ' )
-	    new_name[i]='-';
+	if( connection_name[i]==' ' )
+	    connection_name[i]='-';
 
-	if( new_name[i]=='\n'
-	||  new_name[i]=='\r'
-	||  new_name[i]=='\t' )
+	if( connection_name[i]=='\n'
+	||  connection_name[i]=='\r'
+	||  connection_name[i]=='\t' )
 	{
 	    printf("The specified connection name contains invalid chars.\n");
-	    g_free(new_name);
+	    if( connection_name!=NULL )
+	      g_free(connection_name);
 	    return;
 	}
     }
     if( i < 3 )
     {
 	printf("You must select a longer name for this connection.\n");
-	g_free(new_name);
+	if( connection_name!=NULL )
+	  g_free(connection_name);
 	return;
     }
-    /* Check that the selected connection name is valid. */
-    for(i=0; selected_connection[i]!='\0'; i++)
-    {
-	/* Change spaces to "-" chars */
-	if( selected_connection[i]==' ' )
-	    selected_connection[i]='-';
 
-	if( selected_connection[i]=='\n'
-	||  selected_connection[i]=='\r'
-	||  selected_connection[i]=='\t' )
+    if( selected_connection != NULL )
+    {
+	/* Check that the selected connection name is valid. */
+	for(i=0; selected_connection[i]!='\0'; i++)
 	{
-	    printf("The connection name contains invalid chars.\n");
+	    /* Change spaces to "-" chars */
+	    if( selected_connection[i]==' ' )
+		selected_connection[i]='-';
+
+	    if( selected_connection[i]=='\n'
+	    ||  selected_connection[i]=='\r'
+	    ||  selected_connection[i]=='\t' )
+	    {
+		printf("The connection name contains invalid chars.\n");
+		if( connection_name!=NULL )
+	          g_free(connection_name);
+		return;
+	    }
+	}
+	if( i < 3 )
+	{
+	    printf("You must select a longer name for this connection.\n");
+	    if( connection_name!=NULL )
+	      g_free(connection_name);
 	    return;
 	}
     }
-    if( i < 3 )
-    {
-	printf("You must select a longer name for this connection.\n");
-	return;
-    }
 
-    /* Check if the connection name has changed. If so, move it to the new directory. */
-    if( strcmp((char *)selected_connection, (char *)new_name) !=0 )
+    /* Check that the new connection name doesnt already exist */
+    if( add_new_connection )
     {
-	cmd = g_strdup_printf("mv '%s/client/%s' '%s/client/%s'", OPENVPN_SYSCONF_DIR, selected_connection, OPENVPN_SYSCONF_DIR, new_name);
-	if( ! run_command(cmd) )
+
+	if( connection_exists(widgets, connection_name) )
 	{
-	    printf("Error: could not run command:\n%s\n", cmd);
+	    info = g_strdup_printf("%s", _("A connection with this name already exists.\n"));
+	    show_info(info);
+	    g_free(info);
+	    if( connection_name!=NULL )
+	      g_free(connection_name);
+	    return;
+	}
+    }
+/*
+	path = g_strdup_printf("%s/client/%s", OPENVPN_SYSCONF_DIR, connection_name);
+	if( file_exists(path) )
+	{
+	    info = g_strdup_printf("%s", _("A connection with this name already exists.\n"));
+	    show_info(info);
+	    g_free(info);
+	    if( connection_name!=NULL )
+	      g_free(connection_name);
+	    return;
+	}
+    }
+*/
+
+    if( ! add_new_connection )
+    {
+	/* Check if the connection name has changed. If so, move it to the new directory. */
+	if( strcmp((char *)selected_connection, (char *)connection_name) !=0 )
+	{
+	    cmd = g_strdup_printf("mv '%s/client/%s' '%s/client/%s'", OPENVPN_SYSCONF_DIR, selected_connection, OPENVPN_SYSCONF_DIR, connection_name);
+	    if( ! run_command(cmd) )
+	    {
+		printf("Error: could not run command:\n%s\n", cmd);
+		g_free(cmd);
+		if( connection_name!=NULL )
+	          g_free(connection_name);
+		return;
+	    }
 	    g_free(cmd);
-	    g_free(new_name);
-	    return;
 	}
-	g_free(cmd);
     }
 
     /* Set this connection as the globally selected one */
     if( selected_connection!=NULL )
       g_free(selected_connection);
-    selected_connection = g_strdup_printf("%s", new_name);
+    selected_connection = g_strdup_printf("%s", connection_name);
 
-    g_free(new_name);
+    if( connection_name!=NULL )
+      g_free(connection_name);
 
     /* Create this clients directory. */
     cmd = g_strdup_printf("mkdir -p '%s/client/%s'", OPENVPN_SYSCONF_DIR, selected_connection);
@@ -237,13 +384,6 @@ void apply_client_settings(struct w *widgets)
     /* Remove all "Connection_selected" lines from all confs */
     remove_all_selected_lines();
 
-
-    /* Spin buttons */
-    server_port = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_spinbutton[0]));
-    log_level   = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_spinbutton[1]));
-    proxy_port  = gtk_entry_get_text(GTK_ENTRY(widgets->client_set_spinbutton[2]));
-
-
     /* Combos */
 
     /* Protocol */
@@ -276,29 +416,64 @@ void apply_client_settings(struct w *widgets)
       cipher = g_strdup_printf("cipher DES-EDE3-CBC\n"); /* Triple-DES */
 
 
-
     /* Setup user authentication if the username entry contains a username. */
     if( user!=NULL && strlen(user) > 0 )
     {
-        auth_line = g_strdup_printf("auth-user-pass %s/client/%s/passfile\n", OPENVPN_SYSCONF_DIR, selected_connection);
+        auth_line = g_strdup_printf("auth-user-pass %s\n", passpath);
 
 	/* Write the username and password to the passfile if the password entry contains a password */
-	if( pass!=NULL && strlen(pass) > 5 )
+	if( pass!=NULL && strlen(pass) > 3 )
 	{
 	    /* Encrypt the password and write "UserName\nPassword\n" to the passfile */
-	    // OpenVPN server wants a non-encrypted password.
+	    // OpenVPN client wants a non-encrypted password for now.
 	    // encrypted_pass = encrypt_password(pass);
 
-	    pass_file = g_strdup_printf("%s/client/%s/passfile", OPENVPN_SYSCONF_DIR, selected_connection);
+	    /* Run the mount command if any. Ignore errors. */
+	    if( mountcmd!=NULL && strlen(mountcmd) > 7 )
+	    {	    
+		if( ! run_command((char *)mountcmd) )
+		{
+		    // Dont show an error
+		}
+	    }
+	    
+	    /* Create the passfile directory */
+	    info = g_strdup_printf("%s", passpath);
+
+	    /* Snip a the last '/' from info */
+	    for(i=strlen(info)-1; info[i]!='\0'; i--)
+	    if( info[i]=='/' )
+	    {
+		info[i]='\0';
+		break;
+	    }
+	    /* Check password path validity */
+	    if( strlen(info) < 3 )
+	    {
+		g_free(info);
+		g_free(auth_line);
+		printf("Passfile path too short\n");
+		return;
+	    }
+
+	    if( ! file_exists(info) )
+	    {
+		make_dir_chmod(info, "600");
+	    }
+	    g_free(info);
+
+	    pass_file = g_strdup_printf("%s", passpath);
 	    if((fp=fopen(pass_file, "w+"))==NULL)
 	    {
 		info = g_strdup_printf(_("Can not write the password file: \n%s\n"), pass_file);
 		show_info(info);
 		g_free(info);
 		g_free(pass_file);
-		// Fixme... free stuff
 		return;
 	    }
+
+	    chmod_file(pass_file, "600");
+
 	    pass_lines = g_strdup_printf("%s\n%s\n\n", user, pass);
 	    fputs(pass_lines, fp);
 	    fclose(fp);
@@ -330,7 +505,6 @@ void apply_client_settings(struct w *widgets)
     add_firestarter_user_pre();
 
 
-
     /* Share connection with LAN */
     active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->client_set_combo[4]));
     if( active_index == 0 )
@@ -341,10 +515,9 @@ void apply_client_settings(struct w *widgets)
     {
 	share_tunnel_with_lan = g_strconcat(
 	IP_FORWARD_CMD_START, "\n",
-	"# Needs fixing. Requires LAN addr and subnet mask. Not interfaced yet.\n",
-	"echo \"iptables -t nat -A POSTROUTING -s LAN_ADDR/LAN_SUBN_MASK -o eth0 or tap0\"\n",
 	NULL);
-//	g_free(share_tunnel_with_lan);
+//	"# Needs fixing. Requires LAN addr and subnet mask. Not interfaced yet.\n",
+//	"echo \"iptables -t nat -A POSTROUTING -s LAN_ADDR/LAN_SUBN_MASK -o eth0 or tap0\"\n",
     }
 
     /* Firestarter start and stop script parts if used. */
@@ -416,14 +589,13 @@ void apply_client_settings(struct w *widgets)
     else
       start_at_boot_label = g_strdup_printf("# Client starts at boot\n");
 
-    /* See if we should use a proxy. */
+    /* Use proxy. */
     if( proxy_address!=NULL && strlen(proxy_address) > 0 )
       proxy_line = g_strdup_printf("http-proxy-retry\nhttp-proxy %s %s\n", proxy_address, proxy_port);
     else
       proxy_line = g_strdup_printf("\n");
 
 
-
     /* Write the openvpn client configuration file:
        /etc/gadmin-openvpn/client/SITE/gadmin-openvpn-client.conf */
     openvpn_client_conf = g_strdup_printf("%s/client/%s/gadmin-openvpn-client.conf", OPENVPN_SYSCONF_DIR, selected_connection);
@@ -456,11 +628,11 @@ void apply_client_settings(struct w *widgets)
     "remote ", server_address, " ", server_port, "\n",
     proto, "\n",
     "dev ", tap_dev, "\n",
-    "ca   certs/cacert.pem\n",
-    "cert certs/cert.pem\n",
-    "key  certs/key.pem\n",
-    "dh   certs/dh.pem\n",
-    "tls-auth certs/ta.key 1\n",
+    "ca   certs/", cacert,"\n",
+    "cert certs/", cert,"\n",
+    "key  certs/", key,"\n",
+    "dh   certs/", dhkey,"\n",
+    "tls-auth certs/", takey," 1\n",
     auth_line,
     proxy_line,
     cipher,
@@ -476,12 +648,19 @@ void apply_client_settings(struct w *widgets)
     "mute 20\n",
     NULL);
 
+/*
+    "ca   certs/cacert.pem\n",
+    "cert certs/cert.pem\n",
+    "key  certs/key.pem\n",
+    "dh   certs/dh.pem\n",
+    "tls-auth certs/ta.key 1\n",
+*/
+
     fputs(conf_data, fp);
     fclose(fp);
     g_free(conf_data);
     g_free(openvpn_client_conf);
 
-
     g_free(proto);
     g_free(tap_dev);
     g_free(auth_line);
@@ -513,6 +692,7 @@ void apply_client_settings(struct w *widgets)
     "\n",
     start_at_boot_label,
     "\n",
+    mountcmd,
     "\n",
 //    "lan_eth=\"", lan_iface_name, "\"\n",
 //    "lan_eth_ip=\"", lan_iface_address, "\"\n",
@@ -527,7 +707,7 @@ void apply_client_settings(struct w *widgets)
     "\n",
     "modprobe tun\n",
     "\n",
-    "openvpn --daemon --writepid /var/run/openvpn/openvpn-client.pid \\\n",
+    "openvpn --daemon --script-security 2 --writepid /var/run/openvpn/openvpn-client.pid \\\n",
     "--config ", OPENVPN_SYSCONF_DIR, "/client/", selected_connection, "/gadmin-openvpn-client.conf \\\n",
     "--cd ", OPENVPN_SYSCONF_DIR, "/client/", selected_connection, "\n",
     "\n",
@@ -562,7 +742,7 @@ void apply_client_settings(struct w *widgets)
     g_free(share_tunnel_with_lan);
 
 
-    /* Put the script with the others as: /etc/rc.d/init.d/gadmin-openvpn-client  */
+    /* Put the script with the others as: /etc/init.d/gadmin-openvpn-client  */
     sysinit_script_path = g_strdup_printf("%s/gadmin-openvpn-client", SYSINIT_SCRIPTS_DIR);
     if((fp=fopen(sysinit_script_path, "w+"))==NULL)
     {
