@@ -1,5 +1,5 @@
 /* GAdmin-OpenVPN - An easy to use GTK+ frontend for the openvpn client.
- * Copyright (C) 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2008 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,10 @@
 
 extern struct w *widgets;
 
+int has_value(gchar *input);
+void add_firestarter_user_pre();
+int connection_exists(struct w *widgets, gchar *name);
+void add_connection(struct w *widgets);
 void apply_client_settings(struct w *widgets);
 
 #endif
