@@ -1,5 +1,5 @@
 /* GAdmin-OpenVPN - An easy to use GTK+ frontend for the openvpn client.
- * Copyright (C) 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2008 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,4 +25,8 @@ extern struct w *widgets;
 
 void create_client_settings(struct w *widgets);
 
+char * get_import_filename(gchar *path);
+void copy_import_file(struct w *widgets, gchar *path);
+int valid_connection_name(struct w *widgets);
+
 #endif
