@@ -1,5 +1,5 @@
 /* GAdmin-OpenVPN - An easy to use GTK+ frontend for the openvpn client.
- * Copyright (C) 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2008 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,36 +27,10 @@ extern struct w *widgets;
 
 int cmplowercase(char *buf, char *cmpstring);
 
-//gchar *get_user_setting(gchar *user, gchar *what);
-
-//gchar *get_group_name(gchar *groupnr);
-
 char *encrypt_password(G_CONST_RETURN gchar *password);
 
-//int fix_newlines_in_conf();
-
-//void fix_path(char *path);
-
-//void randomize_username(struct w *widgets);
-
-//void randomize_password(struct w *widgets);
-
-//char *random_string(int len);
-
-//int user_exists(G_CONST_RETURN gchar *username);
-
-//int group_exists(G_CONST_RETURN gchar *username);
-
-//gchar *get_user_comment(gchar *user);
-
-//void password_user(G_CONST_RETURN gchar *username, G_CONST_RETURN gchar *password);
-
-//int is_banned(char *user);
-
-//void exec_ssl(char *command, char descr[128]);
-
 void make_dir_chmod(gchar *directory, char perm[128]);
 
-//int file_exists(char *entity);
+void chmod_file(gchar *filepath, char perm[128]);
 
 #endif
