@@ -1,5 +1,5 @@
 /* GAdmin-OpenVPN - An easy to use GTK+ frontend for the openvpn client.
- * Copyright (C) 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2008 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -356,7 +356,6 @@ void populate_client_settings(struct w *widgets)
     /* Select a row that matches the selected connection name */
     select_row_with_name(selected_connection, widgets);
 
-
     /* Clear all entries */
     for(i=0; i<4; i++)
       gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[i]), "");
@@ -376,6 +375,8 @@ void populate_client_settings(struct w *widgets)
     for(i=0; i<3; i++)
       gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets->client_set_spinbutton[i]), 0);
 
+    /* Set default openvpn port */
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets->client_set_spinbutton[0]), 1194);
 
     if( selected_connection == NULL || strlen(selected_connection) < 3 )
     {
@@ -383,6 +384,13 @@ void populate_client_settings(struct w *widgets)
 	return;
     }
 
+    /* Set default passfile path in the entry */
+    info = g_strdup_printf("%s/client/%s/passfile", OPENVPN_SYSCONF_DIR, selected_connection);
+    utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
+    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[4]), utf8);
+    if( utf8!=NULL )
+      g_free(utf8);
+    g_free(info);
 
     /* Insert values from gadmin-openvpn-client.conf */
     openvpn_client_conf = g_strdup_printf("%s/client/%s/gadmin-openvpn-client.conf", OPENVPN_SYSCONF_DIR, selected_connection);
@@ -475,9 +483,10 @@ void populate_client_settings(struct w *widgets)
 	    }
 	}
 
-	/* Username: auth-user-pass: read the path / passfile */
+	/* Username: auth-user-pass: read the path/passfile */
 	if( cmplowercase(line, "auth-user-pass ") )
 	{
+	    /* Get the path */
 	    for(i=0; line[i]!='\0'; i++)
 	      if( line[i]!=' ')
 	        break;
@@ -490,6 +499,12 @@ void populate_client_settings(struct w *widgets)
 	    if( tmp[strlen(tmp)-1]=='\n')
 	      tmp[strlen(tmp)-1]='\0';
 
+	    /* Insert the path into the passfile entry */
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[4]), utf8);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+
 	    /* Open this path and get the username from the first row in the file */
 	    if((fp1=fopen(tmp, "r"))==NULL)
 	    {
@@ -507,13 +522,14 @@ void populate_client_settings(struct w *widgets)
 		if( file_size1 > 1 )
 		while(fgets(line1, file_size1, fp1)!=NULL)
 		{
-		    if( line1==NULL || strlen(line1) < 4 || commented(line1) )
+		    if( line1==NULL || strlen(line1) < 3 || commented(line1) )
 		      continue;
 
 		    snprintf(tmp1, 1000, "%s", line1);
 		    if( tmp1[strlen(tmp1)-1]=='\n')
 		      tmp1[strlen(tmp1)-1]='\0';
 
+		    /* Insert the username */
 		    utf8 = g_locale_to_utf8(tmp1, strlen(tmp1), NULL, NULL, NULL);
 		    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[2]), utf8);
 		    if( utf8!=NULL )
@@ -657,7 +673,7 @@ void populate_client_settings(struct w *widgets)
 	    tmp[i]='\0';
 
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
-	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[4]), utf8);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[6]), utf8);
 	    if( utf8!=NULL )
 	      g_free(utf8);
 
@@ -676,6 +692,99 @@ void populate_client_settings(struct w *widgets)
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets->client_set_spinbutton[2]), i);
 	    }
 	}
+
+
+	/* Ca cert */
+	if( cmplowercase(line, "ca ") && cmplowercase(line, " certs/")
+	&&  line[0]=='c' && line[1]=='a' && line[2]==' ' )
+	{
+	    for(i=0; line[i]!='\0'; i++)
+	      if( line[i]=='/' )
+	        break;
+
+	    snprintf(tmp, 1000, "%s", &line[i+1]);
+	    if( tmp[strlen(tmp)-1]=='\n' )
+	      tmp[strlen(tmp)-1]='\0';
+
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[7]), utf8);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	/* Cert */
+	if( cmplowercase(line, "cert ") && cmplowercase(line, " certs/")
+	&&  line[0]=='c' && line[1]=='e' && line[2]=='r' && line[3]=='t' && line[4]==' ' )
+	{
+	    for(i=0; line[i]!='\0'; i++)
+	      if( line[i]=='/' )
+	        break;
+
+	    snprintf(tmp, 1000, "%s", &line[i+1]);
+	    if( tmp[strlen(tmp)-1]=='\n' )
+	      tmp[strlen(tmp)-1]='\0';
+
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[8]), utf8);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	/* Key */
+	if( cmplowercase(line, "key ") && cmplowercase(line, " certs/")
+	&&  line[0]=='k' && line[1]=='e' && line[2]=='y' && line[3]==' ' )
+	{
+	    for(i=0; line[i]!='\0'; i++)
+	      if( line[i]=='/' )
+	        break;
+
+	    snprintf(tmp, 1000, "%s", &line[i+1]);
+	    if( tmp[strlen(tmp)-1]=='\n' )
+	      tmp[strlen(tmp)-1]='\0';
+
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[9]), utf8);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	/* DH */
+	if( cmplowercase(line, "dh ") && cmplowercase(line, " certs/")
+	&&  line[0]=='d' && line[1]=='h' && line[2]==' ' )
+	{
+	    for(i=0; line[i]!='\0'; i++)
+	      if( line[i]=='/' )
+	        break;
+
+	    snprintf(tmp, 1000, "%s", &line[i+1]);
+	    if( tmp[strlen(tmp)-1]=='\n' )
+	      tmp[strlen(tmp)-1]='\0';
+
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[10]), utf8);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	/* TLS-AUTH */
+	if( cmplowercase(line, "tls-auth ") && cmplowercase(line, " certs/")
+	&&  line[0]=='t' && line[1]=='l' && line[2]=='s' && line[3]=='-' && line[4]=='a' )
+	{
+	    for(i=0; line[i]!='\0'; i++)
+	      if( line[i]=='/' )
+	        break;
+
+	    snprintf(tmp, 1000, "%s", &line[i+1]);
+	    if( tmp[strlen(tmp)-1]=='\n' )
+	      tmp[strlen(tmp)-1]='\0';
+
+	    /* Snip the last space before "1" */
+	    for(i=0; tmp[i]!='\0'; i++)
+	      if( tmp[i]==' ' )
+	        break;
+	    tmp[i]='\0';
+
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[11]), utf8);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
     }
     fclose(fp);
     free(line);
@@ -742,6 +851,16 @@ void populate_client_settings(struct w *widgets)
 		gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->client_set_combo[5]), 1);
 	    }
 
+	    /* Mount command */
+	    if( cmplowercase(line, "mount ") )
+	    {
+		if( line[strlen(line)-1]=='\n')
+	          line[strlen(line)-1]='\0';
+		utf8 = g_locale_to_utf8(line, strlen(line), NULL, NULL, NULL);
+		gtk_entry_set_text(GTK_ENTRY(widgets->client_set_entry[5]), utf8);
+		if( utf8!=NULL )
+	          g_free(utf8);
+	    }
 
 	    /* LAN interface name */
 /*
